@@ -130,54 +130,6 @@ static struct ccl_rpn_node *ccl_rpn_node_mkbool(struct ccl_rpn_node *l,
     return l;
 }
 
-static struct ccl_rpn_node *ccl_rpn_dup(struct ccl_rpn_node *rpn)
-{
-    struct ccl_rpn_node *n;
-    struct ccl_rpn_attr *attr, **attrp;
-    if (!rpn)
-        return 0;
-    n = ccl_rpn_node_create(rpn->kind);
-    switch (rpn->kind)
-    {
-    case CCL_RPN_AND:
-    case CCL_RPN_OR:
-    case CCL_RPN_NOT:
-        n->u.p[0] = ccl_rpn_dup(rpn->u.p[0]);
-        n->u.p[1] = ccl_rpn_dup(rpn->u.p[1]);
-        break;
-    case CCL_RPN_TERM:
-        n->u.t.term = xstrdup(rpn->u.t.term);
-        n->u.t.qual = rpn->u.t.qual ? xstrdup(rpn->u.t.qual) : 0;
-        attrp = &n->u.t.attr_list;
-        for (attr = rpn->u.t.attr_list; attr; attr = attr->next)
-        {
-            *attrp = (struct ccl_rpn_attr *) xmalloc(sizeof(**attrp));
-            (*attrp)->kind = attr->kind;
-            (*attrp)->type = attr->type;
-            if (attr->kind == CCL_RPN_ATTR_STRING)
-                (*attrp)->value.str = xstrdup(attr->value.str);
-            else
-                (*attrp)->value.numeric = attr->value.numeric;
-            if (attr->set)
-                (*attrp)->set = xstrdup(attr->set);
-            else
-                (*attrp)->set = 0;
-            attrp = &(*attrp)->next;
-        }
-        *attrp = 0;
-        break;
-    case CCL_RPN_SET:
-        n->u.setname = xstrdup(rpn->u.setname);
-        break;
-    case CCL_RPN_PROX:
-        n->u.p[0] = ccl_rpn_dup(rpn->u.p[0]);
-        n->u.p[1] = ccl_rpn_dup(rpn->u.p[1]);
-        n->u.p[2] = ccl_rpn_dup(rpn->u.p[2]);
-        break;
-    }
-    return n;
-}
-
 /**
  * ccl_rpn_delete: Delete RPN tree.
  * rpn:   Pointer to tree.
@@ -634,7 +586,6 @@ static struct ccl_rpn_node *ccl_term_multi_use(CCL_parser cclp,
 }
 
 static struct ccl_rpn_node *split_recur(CCL_parser cclp, ccl_qualifier_t *qa,
-                                        struct ccl_rpn_node *parent,
                                         struct ccl_token **ar, size_t sz)
 {
     size_t l;
@@ -642,7 +593,6 @@ static struct ccl_rpn_node *split_recur(CCL_parser cclp, ccl_qualifier_t *qa,
     assert(sz > 0);
     for (l = 1; l <= sz; l++)
     {
-        struct ccl_rpn_node *p1;
         struct ccl_rpn_node *p2 = ccl_term_multi_use(cclp, ar[0],
                                                      qa, l,
                                                      l > 1,
@@ -652,24 +602,17 @@ static struct ccl_rpn_node *split_recur(CCL_parser cclp, ccl_qualifier_t *qa,
             ccl_rpn_delete(p_top);
             return 0;
         }
-        if (parent)
-        {
-            struct ccl_rpn_node *tmp = ccl_rpn_node_create(CCL_RPN_AND);
-            tmp->u.p[0] = l > 1 ? ccl_rpn_dup(parent) : parent;
-            tmp->u.p[1] = p2;
-            p2 = tmp;
-        }
         if (sz > l)
-            p1 = split_recur(cclp, qa, p2, ar + l, sz - l);
-        else
-            p1 = p2;
-        if (!p1)
         {
-            ccl_rpn_delete(p2);
-            ccl_rpn_delete(p_top);
-            return 0;
+            struct ccl_rpn_node *p1 = split_recur(cclp, qa, ar + l, sz - l);
+            if (!p1)
+            {
+                ccl_rpn_delete(p2);
+                return 0;
+            }
+            p2 = ccl_rpn_node_mkbool(p2, p1, CCL_RPN_AND);
         }
-        p_top = ccl_rpn_node_mkbool(p_top, p1, CCL_RPN_OR);
+        p_top = ccl_rpn_node_mkbool(p_top, p2, CCL_RPN_OR);
     }
     assert(p_top);
     return p_top;
@@ -697,7 +640,7 @@ static struct ccl_rpn_node *search_term_split_list(CCL_parser cclp,
         ar[i] = lookahead;
         lookahead = lookahead->next;
     }
-    p = split_recur(cclp, qa, 0, ar, sz);
+    p = split_recur(cclp, qa, ar, sz);
     xfree(ar);
     for (i = 0; i < sz; i++)
         ADVANCE;
